
using Apaya.Connector.Template.Helpers;
using Apaya.Connector.Template.Models.Charge;
using Apaya.Connector.Core.Actions;
using Apaya.Connector.Core.Actions.CapturePayment;
using Apaya.Connector.Core.Configuration;
using Apaya.Connector.Core.Exceptions;
using Newtonsoft.Json;
using RestSharp;


namespace Apaya.Connector.Template
{
    //Example Capture Action Handler for payment of preAuthorised amount -  based off Core Connector dll
    public class CapturePaymentAction : ICapturePaymentAction
    {
        private static string _connectorName;
        private static string _secretKey;
        private RestClient connRestClient { get; set; }

        public CapturePaymentAction()
        {
            _connectorName = Constants.ConnectorName;
            _secretKey = Constants.SecretKey;
        }


        public async Task<CapturePaymentResponse> CapturePaymentAsync(CapturePaymentRequest request,
            AvailableConnectorConfiguration configuration,
            CancellationToken cancellationToken)
        {
            try
            {
                // see Mapper class
                var command = request.MapCapture();

                // Example - setup headers for connector API call
                var connRequest = new RestRequest(Method.POST);
                connRequest.AddHeader("content-type", Constants.JsonType);
                connRequest.AddHeader("Authorization", $"Bearer {configuration.MerchantConfiguration["SecretKey"]}");

                var settings = new JsonSerializerSettings()
                {
                    ContractResolver = new OrderedContractResolver(),
                    NullValueHandling = NullValueHandling.Ignore
                };

                // Example - Build json payload and Make API call
                connRestClient = new RestClient(Helpers.Environment.GetEnvironmentUrl(ChargeUrl.BaseProductionUrl, ChargeUrl.BaseSandboxUrl));

                var json = JsonConvert.SerializeObject(command, Formatting.Indented, settings);
                connRequest.AddParameter(Constants.JsonType, json, ParameterType.RequestBody);

                var connResponse = await connRestClient.ExecuteAsync(connRequest, cancellationToken);

                //Example - Handle response
                var response =
                    JsonConvert.DeserializeObject<ChargeResponse>(connResponse.Content);

                // see Mapper class
                var resp = response.MapCaptureResponse();

                //captured for logging purposes
                resp.RawActionRequest = new RawConnectorPayload(ConnectorPayloadContentType.Json,
                    new string[] { }, json);

                resp.RawActionResponse = new RawConnectorPayload(ConnectorPayloadContentType.Json,
                    new string[] { }, JsonConvert.SerializeObject(response));

                return resp;
            }
            catch (Exception e)
            {
                // may want to handle specific errors
                throw new GenericConnectorException(e.Message, "500");
            }
        }


        public class CapturePaymentActionResolver : ICapturePaymentActionProvider
        {
            public ICapturePaymentAction GetHandler()
            {
                return new CapturePaymentAction();
            }
        }
    }
}