
using Apaya.Connector.Template.Helpers;
using Apaya.Connector.Template.Models.Charge;
using Apaya.Connector.Template.Models.CheckStatus;
using Apaya.Connector.Core.Actions;
using Apaya.Connector.Core.Actions.GetTransaction;
using Apaya.Connector.Core.Configuration;
using Newtonsoft.Json;
using RestSharp;


namespace Apaya.Connector.Template
{
    //Example GetTransaction / Check Status Action Handler based off Core Connector dll
    public class GetTransactionAction : IGetTransactionAction
    {
        private static string _connectorName;
        private static string _secretKey;

        private RestClient connRestClient { get; set; }

        public GetTransactionAction()
        {
            _connectorName = Constants.ConnectorName;
            _secretKey = Constants.SecretKey;
        }


        public async Task<GetTransactionResponse> GetTransactionAsync(GetTransactionRequest request,
            AvailableConnectorConfiguration configuration, CancellationToken cancellationToken)
        {
            // Validate the configuration holds the required keys for this connector - _secretKey in this case
            DefaultConnectorConfigurationValidator validator = new DefaultConnectorConfigurationValidator();
            validator.ValidateMerchantConfig(_connectorName, configuration, new[] { _secretKey });

            // Example - setup headers for connector API call
            var connRequest = new RestRequest(Method.GET);
            connRequest.AddHeader("content-type", Constants.JsonType);
            connRequest.AddHeader("Authorization", $"Bearer {configuration.MerchantConfiguration["SecretKey"]}");

            // Example - Build json payload and Make API call
            string url = Helpers.Environment.GetEnvironmentUrl(ChargeUrl.BaseProductionUrl, ChargeUrl.BaseSandboxUrl);
            url = $"{url}/{request.TransactionId}";
            connRestClient = new RestClient(url);

            //Example - Handle response
            var connResponse = await connRestClient.ExecuteAsync(connRequest, cancellationToken);

            var response = JsonConvert.DeserializeObject<CheckStatusResponse>(connResponse.Content);

            // see Mapper class
            var resp = response.Map();

            //captured for logging purposes
            resp.RawActionRequest = new RawConnectorPayload(ConnectorPayloadContentType.Json,
                new string[] { }, JsonConvert.SerializeObject(request));
            resp.RawActionResponse = new RawConnectorPayload(ConnectorPayloadContentType.Json,
                new string[] { }, JsonConvert.SerializeObject(response));

            return resp;

            //Would want to add try/catch for specific errors
        }


        public class GetTransactionActionResolver : IGetTransactionActionProvider
        {
            public IGetTransactionAction GetHandler()
            {
                return new GetTransactionAction();
            }
        }
    }
}