using Apaya.Connector.Template.Helpers;
using Apaya.Connector.Template.Models.Void;
using Apaya.Connector.Core.Actions;
using Apaya.Connector.Core.Actions.VoidPayment;
using Apaya.Connector.Core.Configuration;
using Newtonsoft.Json;
using RestSharp;
using Apaya.Connector.Core.Exceptions;

namespace Apaya.Connector.Template
{
    public class VoidPaymentAction : IVoidPaymentAction
    {
        //Example Void Action Handler based off Core Connector dll
        private static string _connectorName;
        private static string _secretKey;
        private RestClient connClient { get; set; }

        public VoidPaymentAction()
        {
            _connectorName = Constants.ConnectorName;
            _secretKey = Constants.SecretKey;

        }


        public async Task<VoidPaymentResponse> VoidPaymentAsync(VoidPaymentRequest request,
            AvailableConnectorConfiguration configuration,
            CancellationToken cancellationToken)
        {
            try
            {

                // Validate the configuration holds the required keys for this connector - _secretKey in this case
                var validator = new DefaultConnectorConfigurationValidator();
                validator.ValidateMerchantConfig(_connectorName, configuration, new[] { _secretKey });

                // see Mapper class
                var command = request.Map();

                // Example - setup headers for connector API call
                var connRequest = new RestRequest(Method.POST);
                connRequest.AddHeader("content-type", Constants.JsonType);
                connRequest.AddHeader("Authorization", $"Bearer {configuration.MerchantConfiguration["SecretKey"]}");

                var settings = new JsonSerializerSettings()
                {
                    ContractResolver = new OrderedContractResolver(),
                    NullValueHandling = NullValueHandling.Ignore
                };

                // Example - Build json payload and Make API call
                connClient = new RestClient(Helpers.Environment.GetEnvironmentUrl(VoidUrl.BaseProductionUrl, VoidUrl.BaseSandboxUrl));

                var json = JsonConvert.SerializeObject(command, Formatting.Indented, settings);
                connRequest.AddParameter(Constants.JsonType, json, ParameterType.RequestBody);

                var connResponse = await connClient.ExecuteAsync(connRequest, cancellationToken);

                //Example - Handle response
                var response = JsonConvert.DeserializeObject<VoidResponse>(connResponse.Content);

                // see Mapper class
                var resp = response.Map();

                //captured for logging purposes
                resp.RawActionRequest = new RawConnectorPayload(ConnectorPayloadContentType.Json,
                    new string[] { }, JsonConvert.SerializeObject(request));

                resp.RawActionResponse = new RawConnectorPayload(ConnectorPayloadContentType.Json,
                    new string[] { }, JsonConvert.SerializeObject(response));

                return resp;

            }
            catch (Exception e)
            {
                // may want to handle specific errors
                throw new GenericConnectorException(e.Message, "500");
            }
        }


        public class VoidPaymentActionResolver : IVoidPaymentActionProvider
        {
            public IVoidPaymentAction GetHandler()
            {
                return new VoidPaymentAction();
            }
        }
    }
}