
using Apaya.Connector.Template.Helpers;
using Apaya.Connector.Template.Webhook;
using Apaya.Connector.Core.Exceptions;
using Apaya.Connector.Core.Webhooks;
using Apaya.Connector.Template.Helpers.SignatureHelper;

namespace Apaya.Connector.Template
{
    // Example Webhook processor implementation - based off Core Connector dll interface
    // Generic connector will receive the initial webhook payload and pass it into the various functions here for processing
    public class WebhookProcessor : IWebhookProcessor
    {
        private readonly List<string> _successEventTypes = new List<string>() { Constants.SUCCESS_CAPTURED };

        public bool CanHandleWebhook(RawWebhook rawWebhook)
        {
            // Example Function to confirm this connector handles  webhooks - pass back agreed connector name
            return rawWebhook.SourceConnector == Constants.ConnectorName;
        }

        public void ValidateWebhookSchema(RawWebhook rawWebhook)
        {
            // Example Function validates schema correct for this connector
            var webhook = getWebhookData(rawWebhook);
            bool isRequestValid = ValidateNotificationRequest(webhook, rawWebhook);

            if (!isRequestValid)
            {
                throw new InvalidWebhookSchemaException($"{Constants.ConnectorName} Schema is invalid");
            }
        }


        public WebhookTransactionIdsResponse GetTransactionIds(RawWebhook rawWebhook)
        {
            // Example Function to get our returned Transaction / Unique Id from webhook payload
            var webhook = getWebhookData(rawWebhook);

            return new WebhookTransactionIdsResponse(webhook.id)
            {
                ApayaTransactionId = long.Parse(webhook.reference.transaction)
            };
        }

        public GetWebhookStatusResponse GetWebhookStatus(RawWebhook rawWebhook)
        {
            // Example Function to get returned Transaction status from webhook payload
            var webhook = getWebhookData(rawWebhook);

            var isSuccess = IsSuccessEvent(webhook.status);

            var token = webhook?.card?.id + "&&" + webhook?.customer?.id;

            return new GetWebhookStatusResponse(webhook.status, isSuccess, token);
        }

        private bool IsSuccessEvent(string connResponseStatus) => _successEventTypes.Contains(connResponseStatus);

        public void ValidateWebhookSignature(RawWebhook rawWebhook, WebhookConfig webhookConfig)
        {
            // If Required, Use to vaiuldate any header signature in the connector webhook payload
            // Some example signature helper code has been provided as examples to assist with this.
            bool isRequestValid = true;
            if (!isRequestValid)
            {
                throw new SignatureException($"{Constants.ConnectorName} Signature is invalid");
            }
        }

        private TemplateWebhookParams getWebhookData(RawWebhook rawWebhook)
        {
            return TemplateWebhookParams.FromRawWebhook(rawWebhook);
        }

        private bool ValidateNotificationRequest(TemplateWebhookParams webhookParams, RawWebhook rawWebhook)
        {
            var validationResult = webhookParams.Validate();
            return validationResult.isValid;
        }
    }
}