using Apaya.Connector.Template;
using Apaya.Connector.Core.Actions.GetTransaction;
using Apaya.Connector.Core.Actions.PaymentMethods;
using Apaya.Connector.Core.Actions.ProcessPayment;
using Apaya.Connector.Core.Actions.ProcessRefund;
using Apaya.Connector.Core.Configuration;
using Apaya.Connector.Core.Entities;
using Apaya.Connector.Core.Webhooks;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using NUnit.Framework.Internal;
using System;

namespace Connector.TestProject
{
    // Example test harness can be used to test the connector API calls
    // Change values / tests to suit your specific testing
    //
    [TestClass()]
    public class GetTransactionActionTests
    {
        [TestMethod()]
        public void GetTransactionAsyncTest()
        {
            GetTransactionAction action = new GetTransactionAction();

            var transRequest = new GetTransactionRequest()
            {
                TransactionId = "280088888"
            };

            var merchantConfig = new Dictionary<string, string>();
            merchantConfig.Add("SecretKey", "sk_test_XXXXXXXXXXXXXXXXXXXXXX");
           // merchantConfig.Add("MerchantId", "123");

            var config = new AvailableConnectorConfiguration()
            {
                MerchantConfiguration = merchantConfig,
                ProductConfiguration = new Dictionary<string, string>(),
                ProviderConfiguration = null
            };

            var response = Task.Run(() => action.GetTransactionAsync(transRequest, config, new CancellationToken())).Result;

            Assert.Fail();
        }

        [TestMethod()]
        public void TestWebhook()
        {
            RawWebhook webhook = new RawWebhook("Template");
            webhook.Body = null;
            webhook.RawRequestContent = "";
            webhook.RawQueryParamsStr = "";
            webhook.Headers = new Dictionary<string, string>()
            {
                { "Content-Type", "application/x-www-form-urlencoded; charset=utf-8" },
                { "Accept-Encoding", "gzip,deflate" },
                { "Host", "fa0b-185-97-92-119.ngrok-free.app" },
                { "User-Agent", "Apache-HttpClient/4.5.12 (Java/1.8.0_352)" },
                { "Content-Length", "485" },
                { "X-Forwarded-For", "23.23.145.89" },
                { "X-Forwarded-Proto", "https" }
            };

            webhook.QueryParams = new Dictionary<string, string>()
            {
                { "amount", "3450" },
                { "response_code", "10078" },
                { "card_number", "444000******0010" },
                { "signature", "4d3ed34bd65edd8a0f3fc0e7661d0626a94d0122455f66a41cd47fafdb921af9" },
                { "merchant_identifier", "0416dde9" },
                { "access_code", "ZWy1WURssNSD62q36ogi" },
                { "order_description", "Apaya T-Shirts" },
                { "payment_option", "VISA" },
                { "expiry_date", "3901" },
                { "customer_ip", "::1" },
                { "language", "en" },
                { "eci", "ECOMMERCE" },
                { "fort_id", "169996200009755966" },
                { "command", "PURCHASE" },
                { "response_message", "failed to check fraud screen" },
                { "merchant_reference", "28003178" },
                { "customer_email", "test@test.com" },
                { "currency", "SAR" },
                { "status", "10" }
            };

            var processor = new WebhookProcessor();

            try
            {
                processor.ValidateWebhookSchema(webhook);

                Assert.IsTrue(true);
            }
            catch
            {
                Assert.Fail();
            }

        }

        [TestMethod()]
        public void ProcessSinglePaymentAsyncTest()
        {
            //request
            var processPaymentRequest = new ProcessPaymentRequest
            {
                ErrorUrl = "https://apaya.io/callback",
                SuccessUrl = "https://apaya.io/callback",
                Order = new Order
                {
                    Discount = 0,
                    Description = "TestProd",
                    TotalAmount = 10.00m,
                    TaxAmount = 1.50m,
                    ShippingAmount = 0,
                    Currency = "AED",
                    Customer = new Customer
                    {
                        Email = "hs@test.com",
                        Phone = null,
                        LastName = null,
                        FirstName = "test",
                        IpAddress = "::1"
                    },
                    BillingAddress = null,
                    ShippingAddress = null,
                    Items = new List<OrderItem>(),
                    ApayaProduct = new ApayaProduct { Id = 93000160 }
                },
                ProviderId = 1003,
                Payment = new PaymentInfo
                {
                    PaymentType = PaymentType.CARD,  //3 or 2 ??
                    TransactionType = TransactionType.Single,
                    Reference = "28008888",
                    PassThrough = "Testttt",
                    ApayaTransactionId = 299999999,
                    PpfpmId = 261,
                    MerchantInitiated = true,
                    PreviousPaymentId = "299999999",
                    Dcb = new PaymentMethodDcbOptions { Pin = null, Mx = null },
                    RequiresPin = false,
                    Card = new PaymentMethodCardOptions
                    {
                        CVV = "100",
                        CardHolder = "X Y",
                        CardNumber = "4299999999999999",
                        ThreeDsEnabled = true,
                        ExpiryMonth = 1,
                        ExpiryYear = 39
                    }
                },
                CapturePayment = false
            };

            //config
            var merchantConfig = new Dictionary<string, string>();
            merchantConfig.Add("SecretKey", "sk_test_XXXXXXXXXXXXXXXXXXXXXX");
            //merchantConfig.Add("MerchantId", "123");

            var config = new AvailableConnectorConfiguration()
            {
                MerchantConfiguration = merchantConfig,
                ProductConfiguration = new Dictionary<string, string>(),
                ProviderConfiguration = null
            };

            var action = new ProcessPaymentAction();

            var response = Task.Run(() => action.ProcessPaymentAsync(processPaymentRequest, config, new CancellationToken())).Result;

            Assert.Fail();
        }

        [TestMethod()]
        public void ProcessRecurringPaymentAsyncTest()
        {
            //request
            var processPaymentRequest = new ProcessPaymentRequest
            {
                ErrorUrl = "https://apaya.io/callback",
                SuccessUrl = "https://apaya.io/callback",
                Order = new Order
                {
                    Discount = 0,
                    Description = "TestProd",
                    TotalAmount = 10.00m,
                    TaxAmount = 1.50m,
                    ShippingAmount = 0,
                    Currency = "AED",
                    Customer = new Customer
                    {
                        Email = "hs@test.com",
                        Phone = null,
                        LastName = null,
                        FirstName = "test",
                        IpAddress = "::1"
                    },
                    BillingAddress = null,
                    ShippingAddress = null,
                    Items = new List<OrderItem>(),
                    ApayaProduct = new ApayaProduct { Id = 93000160 }
                },
                ProviderId = 1003,
                Payment = new PaymentInfo
                {
                    PaymentType = PaymentType.CARD,  //3 or 2 ??
                    TransactionType = TransactionType.Recurrent,
                    Reference = "28008085",
                    PassThrough = "Testttt",
                    ApayaTransactionId = 2999999999,
                    PpfpmId = 261,
                    MerchantInitiated = true,
                    PreviousPaymentId = "card_V3Dv3723957FkXD11fl4q217&&cus_TS06A5520231259o2H41105339",
                    Dcb = new PaymentMethodDcbOptions { Pin = null, Mx = null },
                    RequiresPin = false,
                    Card = new PaymentMethodCardOptions
                    {
                        CVV = "100",
                        CardHolder = "X Y",
                        CardNumber = "4299999999999999",
                        ThreeDsEnabled = true,
                        ExpiryMonth = 1,
                        ExpiryYear = 39
                    }
                },
                CapturePayment = true
            };

            //config
            var merchantConfig = new Dictionary<string, string>();
            merchantConfig.Add("SecretKey", "sk_test_XXXXXXXXXXXXXXXXXXXXXX");
            //merchantConfig.Add("MerchantId", "123");

            var config = new AvailableConnectorConfiguration()
            {
                MerchantConfiguration = merchantConfig,
                ProductConfiguration = new Dictionary<string, string>(),
                ProviderConfiguration = null
            };

            var action = new ProcessPaymentAction();

            var response = Task.Run(() => action.ProcessPaymentAsync(processPaymentRequest, config, new CancellationToken())).Result;

            Assert.Fail();
        }


        [TestMethod()]
        public void TestRefund()
        {
            //request

            var processRefundRequest = new ProcessRefundRequest
            {
                Amount = 52.50m,
                Currency = "AED",
                Reason = "Bad Service",
                ConnectorTransactionId = "chg_TS04A592023132XXXXXXXXXXX"
            };

            //config
            var merchantConfig = new Dictionary<string, string>();
            merchantConfig.Add("SecretKey", "sk_test_XXXXXXXXXXXXXXXXXXXXXX");
            //merchantConfig.Add("MerchantId", "123");

            var config = new AvailableConnectorConfiguration()
            {
                MerchantConfiguration = merchantConfig,
                ProductConfiguration = new Dictionary<string, string>(),
                ProviderConfiguration = null
            };

            var action = new ProcessRefundAction();

            var response = Task.Run(() => action.ProcessRefundAsync(processRefundRequest, config, new CancellationToken())).Result;

            Assert.Fail();
        }


    }
}